<?php

/* создадим таблицу в базе данных

    CREATE TABLE `objects` (
        `id` INT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
        `parse_id` VARCHAR(24) NOT NULL,
        `user_phone` BIGINT(12) UNSIGNED NOT NULL DEFAULT '0',
        `url` VARCHAR(255) NOT NULL,
        `created` INT(10) UNSIGNED NULL DEFAULT NULL,
        `updated` INT(10) UNSIGNED NULL DEFAULT NULL,
        PRIMARY KEY (`id`),
        INDEX `updated` (`updated`)
    )
    COLLATE='utf8_general_ci'
    ENGINE=InnoDB;

*/

set_time_limit(300); // принудительно отключаем скрипт через 5 минут
ini_set('memory_limit', '-1');
ini_set('display_errors', 'on');
error_reporting(E_ALL & ~E_DEPRECATED & ~E_STRICT & ~E_NOTICE);

$debug = false; // отладка
$startTime = microtime(true);

// получаем информацию по последним объектам из БД, если есть 
$lastInfo = getLastInfo();

// параметры для URL, список параметров смотрите в документации
$params = [
    'access-token' => 'aEcS9UfAagInparSiv23aoa_vPzxqWvm', // ваш токен доступа
    //'expand'      => 'region,city,type,section,category,metro,material,rentTime,isNew,rooms,history,phoneProtected', // список дополнительных полей (раскомментируейте, удалите лишние, если требуется)
    'timeStart'   => $lastInfo['time'], // дата начала выборки в формате UNIX-time
    'timeEnd'     => time(), // дата конца выборки в формате UNIX-time (лучше указать, чтобы скрипт выполнялся до текущего времени запуска)
    'sortBy'      => 'updated_asc', // отсортируем по возрастанию даты изменения, для удобста обхода объектов в цикле
    'limit'       => 500
];

// получаем объекты
$result = saveObjects($params, $lastInfo['ids'], $debug);

echo PHP_EOL . (empty($result['errorMessage']) ? '' : 'Error: ' . $result['errorMessage'] . PHP_EOL) . 'Updated: ' . $result['updated'] . PHP_EOL . 'Total time: ' . (microtime(true) - $startTime);

// получаем информацию о последних измененных объектах
function getLastInfo()
{
    $pdo = new PDO('mysql:host=localhost;dbname=inpars;charset=utf8', 'root', '');
    $stmt = $pdo->query('SELECT parse_id, updated FROM objects WHERE updated = (SELECT MAX(updated) FROM objects)'); // получаем объекты с самым поздним временем изменения
    $list = $stmt->fetchAll(PDO::FETCH_KEY_PAIR);

    // отдаем время и id
    if ($list) {
        $result['ids'] = array_keys($list); // получаем id
        $result['time'] = array_shift($list); // получаем время
    } else {
        $result['ids'] = [];
        $result['time'] = time() - 3600; // если это первый запуск и объектов в базе еще нет, то запрашиваем предыдущий час
    }

    return $result;
}

// получаем объекты
function saveObjects($params = [], $ignoreIds = [], $debug = false)
{
    $updated = 0;
    $pdo = new PDO('mysql:host=localhost;dbname=inpars;charset=utf8', 'root', '');

    $response = queryObjects($params, $debug); // получаем объекты
    $errorMessage = empty($response['message']) ? null : $response['message'];
    if (!empty($response['data'])) {
        if ($debug) {
            print_r($response['data']);
            return [
                'updated' => $updated,
                'errorMessage' => $errorMessage
            ];
        }

        // буффер для пакетной вставки
        $buffer['sql'] = [];
        $buffer['model'] = [];

        foreach ($response['data'] as $data) {
            $params['timeStart'] = strtotime($data['updated']); // переназначаем timeStart, для следующего запроса

            $id = $data['id'];

            // игнорируем уже добавленные объекты в цикле
            if (in_array($id, $ignoreIds, true)) {
                continue;
            }

            $buffer['sql'][$id] = '(:parse_id' . $id . ', :user_phone' . $id . ', :url' . $id . ', :created' . $id . ', :updated' . $id . ')';
            $buffer['model'] += [
                ':parse_id' . $id => $data['id'],
                ':user_phone' . $id => $data['phones'] ? implode(',', $data['phones']) : 0,
                ':url' . $id => $data['url'],
                ':created' . $id => strtotime($data['created']),
                ':updated' . $id => strtotime($data['updated']),
            ];
            $ignoreIds[] = $id; // добавляем id, в список игнорируемых
            ++$updated;
        }

        unset($response['data']);

        // пакетная вставка
        if (!empty($buffer)) {
            $sql = 'INSERT INTO `objects` (`parse_id`, `user_phone`, `url`, `created`, `updated`) VALUES ';
            $sql .= implode(', ', $buffer['sql']);
            $stmt = $pdo->prepare($sql);
            $stmt->execute($buffer['model']);
        }

        // если данных больше установленного лимита, то делаем следующий запрос
        if (!empty($response['meta']['totalCount']) && (int)$response['meta']['totalCount'] > (int)$params['limit']) {
            // если количество оставшихся запросов меньше 2
            if (!empty($response['meta']['rateRemaining']) && $response['meta']['rateRemaining'] < 2) {
                sleep(7); // ждать 7 секунд, идельный таймаут для соблюдения лимитов между запросами
            } else {
                sleep(2); // ждать 2 секунды между запросами (дайте серверу передышку)
            }

            $newResult = saveObjects($params, $ignoreIds, $debug); // вызываем себя с новым параметром временем timeStart
            $updated += $newResult['updated'];
        }
    }

    return [
        'updated' => $updated,
        'errorMessage' => $errorMessage
    ];
}

// запрос на получение объектов
function queryObjects($params, $debug = false)
{
    $url = 'https://inpars.ru/api/v2/estate?' . urldecode(http_build_query($params));

    if ($debug) {
        echo $url . PHP_EOL;
    }

    $curl = curl_init();
    curl_setopt_array($curl, [
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_MAXREDIRS => 3,
        CURLOPT_TIMEOUT => 20, // таймаут запроса
        CURLOPT_HTTPHEADER => [
            'Accept: application/json',
        ],
    ]);

    $response = curl_exec($curl);
    $error = curl_error($curl);

    curl_close($curl);

    if (!$error) {
        return json_decode($response, true);
    }

    return null;
}
